#!/bin/bash
# © Copyright [2018 - 2020] Micro Focus or one of its affiliates.
#
# The only warranties for products and services of Micro Focus and its affiliates and licensors
# (“Micro Focus”) are as may be set forth in the express warranty statements accompanying such
# products and services. Nothing herein should be construed as constituting an additional
# warranty. Micro Focus shall not be liable for technical or editorial errors or omissions
# contained herein. The information contained herein is subject to change without notice.
#
# Except as specifically indicated otherwise, this document contains confidential information
# and a valid license is required for possession, use or copying. If this work is provided to the
# U.S. Government, consistent with FAR 12.211 and 12.212, Commercial Computer Software,
# Computer Software Documentation, and Technical Data for Commercial Items are licensed
# to the U.S. Government under vendor's standard commercial license.

#SupportConfig out file
LOG_FILE="oes-cis.log"
TAR_FILE="/var/opt/novell/cis/cis_supportconfig.tar"

#Verifies all RPMs installed by CIS pattern
verify_rpms() {
    echo -e "Verifying the CIS configuration RPMs"
    echo -e "===================================="
    echo -e ""
    for name in oes-cis-config oes-cis-configuration oes-cis-scale
    do
        RPM=$(rpm -q $name)
            echo -e "$RPM"
            rpm --verify $RPM
            rpm -qls $RPM
            echo -e ""
    done

    echo -e "Verifying the CIS Infrasturcture RPMs"
    echo -e "===================================="
    echo -e ""
    for name in oes-cis-envoy oes-cis-zookeeper-docker-image oes-cis-kafka-docker-image oes-cis-opensearch-docker-image oes-cis-dep-config
    do
            RPM=$(rpm -q $name)
            echo -e "$RPM"
            rpm --verify $RPM
        rpm -qls $RPM
            echo -e ""
    done

    echo -e "Verifying the CIS Microservices RPMs"
    echo -e "===================================="
    echo -e ""
    for name in microfocus-cis-auth-image microfocus-cis-repaggregator-image microfocus-cis-metadata-image microfocus-cis-collector-image microfocus-cis-policy-image microfocus-cis-gateway-image microfocus-cis-gateway-image microfocus-cis-repcollector-image microfocus-cis-mgmt-image microfocus-cis-aggregator-image oes-cis-fluentbit-docker-image
    do
        RPM=$(rpm -q $name)
        echo -e "$RPM"
        rpm --verify $RPM
        rpm -qls $RPM
        echo -e ""
    done
}

verify_docker_status() {
    status=`systemctl is-active docker.service` > /dev/null 2>&1

    if [ "$status" != "active" ]; then    
        echo -e "Docker service is inactive."
        exit 1
    fi

    echo -e ""
    echo -e "Docker Version\n"
    echo -e "===================================="
    docker version

    echo -e ""
    echo -e "Docker Info"
    echo -e "===================================="
    docker info

    echo -e ""
    echo -e "Docker Images"
    echo -e "===================================="
    docker images

    echo -e ""
    echo -e "Docker processes"
    echo -e "===================================="
    docker ps --all

    FILES='/etc/sysconfig/docker'
    echo -e ""
    echo -e "Docker config files"
    echo -e "===================================="
    cat $FILES

    echo -e ""
    echo -e "Docker processes info"
    echo -e "===================================="
    for i in $(docker ps -a 2>/dev/null | grep -v "COMMAND" | awk '{print $1}');
    do
        echo -e ""
        echo -e "Docker process info: $i"
        echo -e "===================================="
        docker top $i
        echo -e ""
            echo -e "Docker inspect info: $i"
            echo -e "===================================="
        docker inspect $i
    done

    echo -e ""
    echo -e "Docker logs"
    echo -e "===================================="
    journalctl -u docker | tail -n 200
}

collect_cis_service_configurations() {
    echo -e ""
    echo -e "CIS Configuration file { /etc/opt/novell/cis/config } "
    echo -e "====================================================="
    cat /etc/opt/novell/cis/config

    echo -e ""
    echo -e "CIS Configuration status file { /etc/opt/novell/cis/configurationStatus.json }"
    echo -e "=============================================================================="
    cat /etc/opt/novell/cis/configurationStatus.json

    echo -e ""
    echo -e "CIS Datascale Configuration files"
    echo -e "============================================"
    echo -e "{File: /etc/opt/novell/cis-scale/config}"
    cat /etc/opt/novell/cis-scale/config

    echo -e ""
    echo -e "{File: /etc/opt/novell/cis-scale/scale_config}"
    cat /etc/opt/novell/cis-scale/scale_config

    echo -e ""
    echo -e "CIS Infrastructure High Availablity services"
    echo -e "============================================"
    status=`systemctl is-active -l oes-cis-dependencies-ha.service`

    if [ "$status" == "active" ]; then
        echo -e ""
        echo -e "Service log"
        echo -e "============================================"
        journalctl -u oes-cis-dependencies-ha.service | tail -n 200
    else
        echo -e "CIS HA Dependencies service is not running."
    fi

    echo -e ""
    status=`systemctl is-active -l oes-cis-zk.service`
    if [ "$status" == "active" ]; then
        echo -e ""
        echo -e "Zookeeper config file"
        echo -e "============================================"
        cat /etc/opt/novell/cis/zk/config
        echo -e ""
        echo -e "Service log"
        echo -e "============================================"
        journalctl -u oes-cis-zk.service | tail -n 200
    else
        echo -e "CIS Zookeeper service is not running."
    fi

    echo -e ""
    status=`systemctl is-active -l oes-cis-kafka.service`
    if [ "$status" == "active" ]; then
        echo -e ""
        echo -e "Kafka config file"
        echo -e "============================================"
        cat /etc/opt/novell/cis/kafka/config
        echo -e ""
        echo -e "Service log"
        echo -e "============================================"
        journalctl -u oes-cis-kafka.service | tail -n 200
    else
        echo -e "CIS Kafka service is not running."
    fi

    echo -e ""
    status=`systemctl is-active -l oes-cis-opensearch.service`
    if [ "$status" == "active" ]; then
        echo -e ""
        echo -e "opensearch config file"
        echo -e "============================================"
        cat /etc/opt/novell/cis/analytics/config
        echo -e ""
        echo -e "Service log"
        echo -e "============================================"
        journalctl -u oes-cis-opensearch.service | tail -n 200
    else
        echo -e "CIS opensearch service is not running."
    fi

    echo -e ""
    status=`systemctl is-active -l oes-cis-configuration.service`
    if [ "$status" == "active" ]; then
        echo -e ""
        echo -e "Service log"
        echo -e "============================================"
        journalctl -u oes-cis-configuration.service | tail -n 200
    else
        echo -e "CIS Configuration service is not running."
    fi

    echo -e ""
    status=`systemctl is-active -l oes-cis-collector.service`
    if [ "$status" == "active" ]; then
        echo -e ""
        echo -e "Service log"
        echo -e "============================================"
        journalctl -u oes-cis-collector.service | tail -n 200
    else
        echo -e "CIS Collector service is not running."
    fi

    echo -e ""
    status=`systemctl is-active -l oes-cis-aggregator.service`
    if [ "$status" == "active" ]; then
        echo -e ""
        echo -e "Service log"
        echo -e "============================================"
        journalctl -u oes-cis-aggregator.service | tail -n 200
    else
        echo -e "CIS Aggregator service is not running."
    fi

    echo -e ""
    status=`systemctl is-active -l oes-cis-auth.service`
    if [ "$status" == "active" ]; then
        echo -e ""
        echo -e "Service log"
        echo -e "============================================"
        journalctl -u oes-cis-auth.service | tail -n 200
    else
        echo -e "CIS Authorization service is not running."
    fi

    echo -e ""
    status=`systemctl is-active -l oes-cis-data.service`
    if [ "$status" == "active" ]; then
        echo -e ""
        echo -e "Service log"
        echo -e "============================================"
        journalctl -u oes-cis-data.service | tail -n 200
    else
        echo -e "CIS Data service is not running."
    fi

    echo -e ""
    status=`systemctl is-active -l oes-cis-fluentbit.service`
    if [ "$status" == "active" ]; then
        echo -e ""
        echo -e "Service log"
        echo -e "============================================"
        journalctl -u oes-cis-fluentbit.service | tail -n 200
    else
        echo -e "CIS Fluentbit service is not running."
    fi

    echo -e ""
    status=`systemctl is-active -l oes-cis-gateway.service`
    if [ "$status" == "active" ]; then
        echo -e ""
        echo -e "Service log"
        echo -e "============================================"
        journalctl -u oes-cis-gateway.service | tail -n 200
    else
        echo -e "CIS Gateway service is not running."
    fi

    echo -e ""
    status=`systemctl is-active -l oes-cis-metadata.service`
    if [ "$status" == "active" ]; then
        echo -e ""
        echo -e "Service log"
        echo -e "============================================"
        journalctl -u oes-cis-metadata.service | tail -n 200
    else
        echo -e "CIS Metadata service is not running."
    fi

    echo -e ""
    status=`systemctl is-active -l oes-cis-mgmt.service`
    if [ "$status" == "active" ]; then
        echo -e ""
        echo -e "Service log"
        echo -e "============================================"
        journalctl -u oes-cis-mgmt.service | tail -n 200
    else
        echo -e "CIS Management service is not running."
    fi

    echo -e ""
    status=`systemctl is-active -l oes-cis-policy.service`
    if [ "$status" == "active" ]; then
        echo -e ""
        echo -e "Service log"
        echo -e "============================================"
        journalctl -u oes-cis-policy.service | tail -n 200
    else
        echo -e "CIS Policy service is not running."
    fi

    echo -e ""
    status=`systemctl is-active -l oes-cis-repcollector.service`
    if [ "$status" == "active" ]; then
        echo -e ""
        echo -e "Service log"
        echo -e "============================================"
        journalctl -u oes-cis-repcollector.service | tail -n 200
    else
        echo -e "CIS Reporting collector service is not running."
    fi

    echo -e ""
    status=`systemctl is-active -l oes-cis-repaggregator.service`
    if [ "$status" == "active" ]; then
        echo -e ""
        echo -e "Service log"
        echo -e "============================================"
        journalctl -u oes-cis-repaggregator.service | tail -n 200
    else
        echo -e "CIS Reporting Aggregator service is not running."
    fi

    echo -e ""
    status=`systemctl is-active -l oes-cis-dataatscale.service`
    if [ "$status" == "active" ]; then
        echo -e ""
        echo -e "Service log"
        echo -e "============================================"
        journalctl -u oes-cis-dataatscale.service | tail -n 200
    else
        echo -e "CIS Data scale service is not running."
    fi

    echo -e ""
    status=`systemctl is-active -l oes-cis-gatewayforscale.service`
    if [ "$status" == "active" ]; then
        echo -e ""
        echo -e "Service log"
        echo -e "============================================"
        journalctl -u oes-cis-gatewayforscale.service | tail -n 200
    else
        echo -e "CIS Gateway scale service is not running."
    fi

    echo -e ""
    status=`systemctl is-active -l oes-cis-updateiptables.service`
    if [ "$status" == "active" ]; then
        echo -e ""
        echo -e "Service log"
        echo -e "============================================"
        journalctl -u oes-cis-updateiptables.service | tail -n 200
    else
        echo -e "CIS IP Tables service is not running."
    fi
}

echo -e "===================================================================="
echo -e "Cloud Integrated Storage - SupportConfig utility"
echo -e "====================================================================="

echo -e "Validating Cloud Integrated Storage Pattern installation"
#Check if CIS pattern has been installed
zypper se -t pattern | grep cloud-integrated-storage | grep ^i &>/dev/null
if [ $? -ne 0 ]; then
	echo -e "Cloud Integrated Storage pattern not installed"
	exit 0
fi

echo -e "Gathering installed RPMs information"
verify_rpms > $LOG_FILE

echo -e "Gathering Docker deamon information"
verify_docker_status >> $LOG_FILE

echo -e "Gathering CIS services information"
collect_cis_service_configurations >> $LOG_FILE

echo -e "Archiving SupportConfig logs"
if [ -f $TAR_FILE ]; then
    rm $TAR_FILE
fi
tar -jcf $TAR_FILE $LOG_FILE

rm $LOG_FILE
echo -e "CIS SupportConfig run completed. Logs are stored at $TAR_FILE"
exit 0
