#!/bin/bash
# © Copyright [2018 - 2020] Micro Focus or one of its affiliates.
#
# The only warranties for products and services of Micro Focus and its affiliates and licensors
# (“Micro Focus”) are as may be set forth in the express warranty statements accompanying such
# products and services. Nothing herein should be construed as constituting an additional
# warranty. Micro Focus shall not be liable for technical or editorial errors or omissions
# contained herein. The information contained herein is subject to change without notice.
#
# Except as specifically indicated otherwise, this document contains confidential information
# and a valid license is required for possession, use or copying. If this work is provided to the
# U.S. Government, consistent with FAR 12.211 and 12.212, Commercial Computer Software,
# Computer Software Documentation, and Technical Data for Commercial Items are licensed
# to the U.S. Government under vendor's standard commercial license.

while [ "$1" != "" ]; do
case $1 in
        -type )        shift
                       request_type=$1
                       ;;
        -is_secure )   shift
                       is_secure=$1
                       ;;
        -port )        shift
                       port_no=$1
                       ;;
        -username )    shift
                       user_name=$1
                       ;;
        -pwd )         shift
                       password=$1
                       ;;
        -server_ip )      shift
                       server_ip=$1
                       ;;
        -fresh )       shift
                       fresh=$1
                       ;;
        * )            >&2 echo "Invalid option"
					   exit 1
					   ;;

    esac
    shift
done




case $request_type in

        database )
						echo "DB CONF"		
						set -o errexit # abort on nonzero exit status
						set -o nounset # abort on unused variable

						#Function definitions
						usage() {
cat << _EOF_
Usage: ${0} "ROOT PASSWORD"
  "ROOT PASSWORD" is required to configure the DB.
_EOF_
}

						# Returns exit status 0 if the database root password
						# is set, a nonzero exit status otherwise.
						is_mysql_root_password_set() {
						  ! mysqladmin --user=root status > /dev/null 2>&1
						}

						# Returns exit status 0 if the mysql(1) command is available,
						# nonzero exit status otherwise.
						is_mysql_command_available() {
						  which mysql > /dev/null 2>&1
						}

						db_root_password=$password

						# enable remote connection for mysql
						sed -i -e '/bind-address/s/127.0.0.1/0.0.0.0/' /etc/my.cnf

						#Restart mysql service
						systemctl restart mysql.service
						if [ $? -eq 0 ]; then
							echo "mysql service restarted successfully"
						else
							echo "mysql service restart failed"
						fi

						#Enable mysql service
						systemctl reenable mysql.service
						if [ $? -eq 0 ]; then
							echo "mysql service reenabled successfully"
						else
							echo "mysql service reenable failed"
						fi

						#Check mysql service status
						status=`systemctl is-active mysql.service`
						if [ "$status" == "active" ]; then
							echo "mysql service status checked successfully"
						else
							echo "mysql service status check failed"
						fi

						if ! is_mysql_command_available; then
						  >&2 echo "The MariaDB client is not installed."
						  exit 1
						fi

						if is_mysql_root_password_set; then
						  >&2 echo "Database root password is set already"
						  exit 0
						fi


mysql --user=root <<_EOF_
  SET PASSWORD FOR 'root'@localhost = PASSWORD('${db_root_password}');
  DELETE FROM mysql.user WHERE User='';
  DELETE FROM mysql.user WHERE User='root' AND Host NOT IN ('localhost', '127.0.0.1', '::1', '%');
  DROP DATABASE IF EXISTS test;
  DELETE FROM mysql.db WHERE Db='test' OR Db='test\\_%';
  GRANT ALL PRIVILEGES ON *.* TO 'root'@'%' IDENTIFIED BY '${db_root_password}';
  FLUSH PRIVILEGES;
_EOF_
		           ;;
				   
        zk )       echo "ZK  CONF"
                       if ! { [ -e /etc/opt/novell/cis/certs/servercert.pem ] && [ -e /etc/opt/novell/cis/certs/serverkey.pem ] && [ -e /etc/opt/novell/cis/certs/rootCAs/SSCert.pem ]; }; then
                          >&2 echo "Server certificates not found"
                          exit 1;
                       fi

                        systemctl enable oes-cis-zk.service
						            systemctl restart oes-cis-zk.service
						
                        if [ `systemctl is-active oes-cis-zk.service` != "active" ]; then
                               >&2 echo "Zookeeper configuration failed\n" 
                               exit 1
                        fi
                   ;;
				   
				   
        kafka)     echo "Kafka conf";
					   kafka_dir='/etc/opt/novell/cis/kafka'
					   if [ ! -d $kafka_dir ]; then
							>&2 echo "cis-kafka directory not found" 
							exit 1;
					   fi

					   zk_uri=$server_ip:12181
					   config_file='/etc/opt/novell/cis/kafka/config'
					   sed -i -e"s/^MQ_ADVERTISED_ADDRESS=.*/MQ_ADVERTISED_ADDRESS=\"$server_ip\"/" $config_file
					   sed -i -e"s/^MQ_PORT=.*/MQ_PORT=\"$port_no\"/" $config_file
					   sed -i -e"s/^MQ_ZOOKEEPER_URI=.*/MQ_ZOOKEEPER_URI=\"$zk_uri\"/" $config_file
					   sed -i -e"s/^SECURE_MODE=.*/SECURE_MODE=\"$is_secure\"/" $config_file
					   if [ $is_secure ];then
                           if ! { [ -e /etc/opt/novell/cis/certs/servercert.pem ] && [ -e /etc/opt/novell/cis/certs/serverkey.pem ] && [ -e /etc/opt/novell/cis/certs/rootCAs/SSCert.pem ]; }; then
                              >&2 echo "Server certificates not found"
                              exit 1;
                           fi

                            config_file=/etc/opt/novell/cis/kafka/config
                            password=`head /dev/urandom | tr -dc A-Za-z0-9 | head -c 13 ; echo ''`

                            if [ ! -d /etc/opt/novell/cis/kafka/certs ]; then
                                >&2 echo "certs folder not found"
                                exit 1;
                            fi

                            rm -f /etc/opt/novell/cis/kafka/certs/*

                            openssl pkcs12 -export -in /etc/opt/novell/cis/certs/servercert.pem -inkey /etc/opt/novell/cis/certs/serverkey.pem -out /etc/opt/novell/cis/kafka/certs/server.p12 -name localhost -CAfile /etc/opt/novell/cis/certs/rootCAs/SSCert.pem -caname rootca -password pass:$password;
                            if [ $? -ne 0 ]; then
                                >&2 echo "Failed to import the server certificate, private key and CA certificate into PKCS12 keystore"
                                exit 1;
                            fi                                    		   
                            
                            cmdimport=""
                            cmdkeystore=""

                            if [ $fresh="true" ]; then
                              cmdimport="keytool -J-Dkeystore.pkcs12.legacy -importkeystore -deststorepass $password -destkeypass $password -destkeystore /etc/opt/novell/cis/kafka/certs/kafka.keystore.jks -srckeystore /etc/opt/novell/cis/kafka/certs/server.p12 -srcstoretype PKCS12 -srcstorepass $password -alias localhost"
                              cmdkeystore="keytool -J-Dkeystore.pkcs12.legacy -keystore /etc/opt/novell/cis/kafka/certs/kafka.truststore.jks -storepass $password  -alias CARoot -import -file /etc/opt/novell/cis/certs/rootCAs/SSCert.pem -noprompt"
                            else
                              cmdimport="keytool -importkeystore -deststorepass $password -destkeypass $password -destkeystore /etc/opt/novell/cis/kafka/certs/kafka.keystore.jks -srckeystore /etc/opt/novell/cis/kafka/certs/server.p12 -srcstoretype PKCS12 -srcstorepass $password -alias localhost"
                              cmdkeystore="keytool -keystore /etc/opt/novell/cis/kafka/certs/kafka.truststore.jks -storepass $password  -alias CARoot -import -file /etc/opt/novell/cis/certs/rootCAs/SSCert.pem -noprompt"
                            fi
			                      `$cmdimport` > /dev/null 2>&1
                            if [ $? -ne 0 ]; then
                                >&2 echo "Failed to Covert the PKCS12 type keystore to JKS type keystore"
                                exit 1;
                            fi		

                            `$cmdkeystore` > /dev/null 2>&1
                            if [ $? -ne 0 ]; then
                                >&2 echo "Failed to import the CA certificate into JKS type keystore"
                                exit 1;
                            fi
                            sed -i -e"s/^MQ_KEYSTORE_PLAINTEXT_PASSWORD=.*/MQ_KEYSTORE_PLAINTEXT_PASSWORD=\"$password\"/" $config_file
                            if [ $? -ne 0 ]; then
                                >&2 echo "Failed to write MQ_KEYSTORE_PLAINTEXT_PASSWORD in kafka config file"
                                exit 1;
                            fi

                            sed -i -e"s/^MQ_KEY_PLAINTEXT_PASSWORD=.*/MQ_KEY_PLAINTEXT_PASSWORD=\"$password\"/" $config_file
                            if [ $? -ne 0 ]; then
                                >&2 echo "Failed to write MQ_KEYSTORE_PLAINTEXT_PASSWORD in kafka config file"
                                exit 1;
                            fi

                            sed -i -e"s/^MQ_TRUSTSTORE_PLAINTEXT_PASSWORD=.*/MQ_TRUSTSTORE_PLAINTEXT_PASSWORD=\"$password\"/" $config_file
                            if [ $? -ne 0 ]; then
                                >&2 echo "Failed to write MQ_KEYSTORE_PLAINTEXT_PASSWORD in kafka config file"
                                exit 1;
                            fi
                            chmod 0776 /etc/opt/novell/cis/kafka/kafka-logs/
                            chmod 0777 /etc/opt/novell/cis/kafka/certs/server.p12
                           systemctl restart oes-cis-kafka.service                           
                           if [ `systemctl is-active oes-cis-kafka.service` != "active" ]; then
                               >&2 echo "Kafka configuration failed\n"
                               exit 1
                           fi
					   else
							systemctl restart oes-cis-kafka.service							
							if [ `systemctl is-active oes-cis-kafka.service` != "active" ]; then
							   >&2 echo "Kafka configuration failed\n"
							   exit 1
							fi
					   fi

                       systemctl enable oes-cis-kafka.service
			   echo "kafka configured succesfully"
			   ;;
				   
        os)        echo "Opensearch";
                   if [ $is_secure = "yes" ];then
				       config_file='/etc/opt/novell/cis/analytics/config'
					   if ! { [ -e /etc/opt/novell/cis/certs/servercert.pem ] && [ -e /etc/opt/novell/cis/certs/serverkey.pem ] && [ -e /etc/opt/novell/cis/certs/rootCAs/SSCert.pem ]; }; then
                          >&2 echo "Server certificates not found"
						  exit 1;
                       fi

                       sed -i -e"s/^SECURE_MODE=.*/SECURE_MODE=\"$is_secure\"/" $config_file
                   else
                       config_file='/etc/opt/novell/cis/analytics/config'
                       sed -i -e"s/^SECURE_MODE=.*/SECURE_MODE=\"$is_secure\"/" $config_file
                   fi

                   systemctl restart oes-cis-opensearch.service				   
                   if [ `systemctl is-active oes-cis-opensearch.service` != "active" ]; then
                         >&2 echo "Opensearch configuration failed\n" 
                         exit 1
                   fi

                   systemctl enable oes-cis-opensearch.service
                    echo "Succesfully configured Opensearch"

                   ;;
				   
		firewall)  echo "firewall"
          firewall-cmd --query-masquerade --permanent > /dev/null 2>&1
          if [ $? != 0 ]; then
            firewall-cmd --add-masquerade --permanent > /dev/null 2>&1
            if [ $? -ne 0 ]; then
              >&2 echo "Failed to add masquerade in firewall" 
              exit 1;
					  fi
          fi
          for port in 8343 8344 3306 9400 24224; do
            firewall-cmd --permanent --zone=$DEFAULT_ZONE --query-port=${port}/tcp > /dev/null 2>&1
            if [ $? != 0 ]; then
              firewall-cmd --permanent --zone=$DEFAULT_ZONE --add-port=${port}/tcp > /dev/null 2>&1
              if [ $? -ne 0 ]; then
                >&2 echo "Failed to add cis ports in firewall"
                exit 1;
              fi
            fi
          done
          firewall-cmd --reload > /dev/null 2>&1
          if [ $? -ne 0 ]; then
                >&2 echo "Firewall failed to reload"
                exit 1;
          fi
				    ;;

		* ) 	>&2 echo "invalid dependency type"
				exit 1;
		        
esac
