#!/bin/bash
# © Copyright [2016 - 2020] Micro Focus or one of its affiliates.
#
# The only warranties for products and services of Micro Focus and its affiliates and licensors
# (“Micro Focus”) are as may be set forth in the express warranty statements accompanying such
# products and services. Nothing herein should be construed as constituting an additional
# warranty. Micro Focus shall not be liable for technical or editorial errors or omissions
# contained herein. The information contained herein is subject to change without notice.
#
# Except as specifically indicated otherwise, this document contains confidential information
# and a valid license is required for possession, use or copying. If this work is provided to the
# U.S. Government, consistent with FAR 12.211 and 12.212, Commercial Computer Software,
# Computer Software Documentation, and Technical Data for Commercial Items are licensed
# to the U.S. Government under vendor's standard commercial license.

echoerr()
{
	echo "$@" 1>&2;
}

if [ $# != 5 ] ; then
	usage="$(basename "$0") edirip ncpport treename username password"
	echoerr "Not enough arguments"
	echoerr "$usage"
	exit -1
fi

openssl_cmd=/usr/bin/openssl
cert_gen_bin=/opt/novell/cis/bin/signedirchain
cn="cischaincert"

#Get subject name from existing oes server certificate and replace cn with our cn
certsub=$($openssl_cmd x509 -noout -subject -in /etc/ssl/servercerts/servercert.pem | awk '{ print "-subj "$2}' | sed -n 's/CN=.*/CN='"$cn"'/p')
random=$(cat /dev/urandom | tr -cd 'a-f0-9' | head -c 32)
tmp_dir=/var/run/cis/$random

chaincsr_file=$tmp_dir"/chaincacert.csr"
chaincacert_der_file=$tmp_dir"/chaincacert.der"
edircacert_der_file=$tmp_dir"/edircacert.der"
chainkey_file=$tmp_dir"/chaincacertkey.pem"
chaincacert_pem_file=$tmp_dir"/chaincacert.pem"
edircacert_pem_file=$tmp_dir"/SSCert.pem"
chainca_dir=/etc/opt/novell/cis/certs/rootCAs

#If final directory not exist create it
if [ ! -d "$chainca_dir" ]; then
	/usr/bin/mkdir -p $chainca_dir
	if [ $? != 0 ] ; then
		echoerr "Failed to create directory $chainca_dir"
		exit -1
	fi
fi

# create temporary directory
/usr/bin/mkdir -p $tmp_dir
if [ $? != 0 ] ; then
	echoerr "Failed to create temp directory $tmp_dir"
	exit -1
fi

#Create csr file
echo "Creating chain ca csr..."
#Normal log is redirected to stderr, hence redirecting to stdout
$openssl_cmd req -nodes $certsub -newkey rsa:2048 -keyout $chainkey_file -out $chaincsr_file 2>&1
if [ $? != 0 ] ; then
	echoerr "Failed to create chain certificate csr file"
	exit -1
fi

echo "Creating chain certificate using edir..."
$cert_gen_bin $1 $2 $3 $4 $5 $chaincsr_file $chaincacert_der_file $edircacert_der_file
if [ $? != 0 ] ; then
	echoerr "Failed to create chain ca from edirectory"
	exit -1
fi

echo "Converting chain cert from der to pem format..."
$openssl_cmd x509 -inform der -in $chaincacert_der_file -out $chaincacert_pem_file
if [ $? != 0 ] ; then
	echoerr "Failed to convert chain ca cert from der to pem format"
	exit -1
fi

echo "Converting edir root ca from der to pem format..."
$openssl_cmd x509 -inform der -in $edircacert_der_file -out $edircacert_pem_file
if [ $? != 0 ] ; then
	echoerr "Failed to convert edir root ca cert from der to pem format"
	exit -1
fi

echo "Copy ca cert to standard location..."
cp $chainkey_file $chaincacert_pem_file $edircacert_pem_file $chainca_dir
if [ $? != 0 ] ; then
	echoerr "Failed to copy data to $chainca_dir location"
	exit -1
fi
rm -rf $tmp_dir
echo "Done."
exit 0
