#!/bin/bash
# © Copyright [2016 - 2020] Micro Focus or one of its affiliates.
#
# The only warranties for products and services of Micro Focus and its affiliates and licensors
# (“Micro Focus”) are as may be set forth in the express warranty statements accompanying such
# products and services. Nothing herein should be construed as constituting an additional
# warranty. Micro Focus shall not be liable for technical or editorial errors or omissions
# contained herein. The information contained herein is subject to change without notice.
#
# Except as specifically indicated otherwise, this document contains confidential information
# and a valid license is required for possession, use or copying. If this work is provided to the
# U.S. Government, consistent with FAR 12.211 and 12.212, Commercial Computer Software,
# Computer Software Documentation, and Technical Data for Commercial Items are licensed
# to the U.S. Government under vendor's standard commercial license.

echoerr()
{
        echo "$@" 1>&2;
}

if [ $# != 3 ] ; then
        usage="$(basename "$0") cn san tmpdirectory"
        echoerr "Not enough arguments"
        echoerr "$usage"
        exit -1
fi

openssl_cmd=/usr/bin/openssl
#cn="cischaincerti"
#san="subjectAltName=IP:164.99.113.57,IP:192.168.73.120,IP:192.168.73.121\n"
#Get subject name from existing oes server certificate and replace cn with our cn
certsub=$($openssl_cmd x509 -noout -subject -in /etc/ssl/servercerts/servercert.pem | awk '{ print "-subj "$2}' | sed -n 's/CN=.*/CN='"$1"'/p')
dirname=$3

ssl_ext_file=$dirname"/ssl-extensions-x509.cnf"
certcsr_file=$dirname"/servercert.csr"
#certkey_file=$dirname"/serverkey.pem"
certkey_file="/etc/opt/novell/cis/certs/serverkey.pem"
chainca_file="/etc/opt/novell/cis/certs/rootCAs/chaincacert.pem"
chainca_key_file="/etc/opt/novell/cis/certs/rootCAs/chaincacertkey.pem"
servercert_inter_pem_file=$dirname"/servercert_inter.pem"
servercert_pem_file=$dirname"/servercert.pem"
export RANDFILE=$dirname/.rnd

# create temporary directory
/usr/bin/mkdir -p $dirname
if [ $? != 0 ] ; then
        echoerr "Failed to create temp directory $dirname"
        exit -1
fi

#Prepare extension file
printf "[v3_ca]\nbasicConstraints=CA:FALSE\nkeyUsage=digitalSignature,keyEncipherment\nsubjectAltName=$2" > $ssl_ext_file

#Create server csr request file
echo "Creating server csr request file..."
$openssl_cmd req -new -sha256 $certsub -nodes -out $certcsr_file -key $certkey_file
if [ $? != 0 ] ; then
        echoerr "Failed to create server cert csr request"
        exit -1
fi

#TODO how to handle serial numbers - does it really matters in our cis case?
echo "Creating server certificate..."
$openssl_cmd x509 -req -days 730 -in $certcsr_file -CA $chainca_file -CAkey $chainca_key_file -CAcreateserial -out $servercert_inter_pem_file -extensions v3_ca -extfile $ssl_ext_file 2>&1
if [ $? != 0 ] ; then
        echoerr "Failed to create server certificate"
        exit -1
fi

echo "Creating final server certificate with chain ca..."
cat $servercert_inter_pem_file $chainca_file > $servercert_pem_file
if [ $? != 0 ] ; then
        echoerr "Failed to create final server certificate"
        exit -1
fi
echo "Done..."
exit 0
