function postMsg(target_win, msg) {
    target_win[window['postMessage'] ? 'postMessage' : 'getMsg'](msg, "*");
}

function getMsg(event) {
    var res = event.data ? event.data.split("|") : event.split("|");
    if (res[0] === 'updateToc') {
        NAV_MODULE && NAV_MODULE.updateToc(res[1], res[2]);
    } else if (res[0] === 'getIndexTopicsFound' && INDEX_MODULE && INDEX_MODULE.updateTopicsFoundList) {
        /*
         This message is from topicfound.html in the topic window indicating
         it's ready to receive the topics found list from the index.
         */
        INDEX_MODULE.updateTopicsFoundList();
    }
}

var DOM_UTILS_MODULE = new function () {
    this.loadJSFile = function (filename) {
        var script = document.createElement("script");
        script.type = "text/javascript";
        /*
         readystate or onload not completely reliable and/or not supported in older browsers
         instead we'll have each js file call back at last line
       */
        script.src = filename;
        document.getElementsByTagName("head")[0].appendChild(script);
    };

    this.getEl = function (doc, objName) {
        return doc.getElementById(objName);
    };

    this.getScrollY = function () {
        if (typeof (window.pageYOffset) == 'number') {//Netscape compliant
            return window.pageYOffset;
        } else if (document.body && document.body.scrollTop) { //DOM compliant
            return document.body.scrollTop;
        } else if (document.documentElement && document.documentElement.scrollTop) {//IE6 standards compliant mode
            return document.documentElement.scrollTop;
        }

        return 0;
    };

    this.getClientHeight = function () {
        if (typeof (window.innerWidth) == 'number') {
            return window.innerHeight;
        } else if (document.documentElement && document.documentElement.clientHeight) { //IE 6+ standards
            return document.documentElement.clientHeight;
        } else if (document.body && document.body.clientHeight) { //IE 4 compatible
            return document.body.clientHeight;
        }

        return 0;//?
    };
}();

var TOC_MODULE;
TOC_MODULE = new function (dutils) {
    var theToc = null;

    this.getToc = function () {
        if (!theToc) {
            theToc = new Toc();
        }

        return theToc;
    };

    function ImgStoreObject() {
        this.count = -1;
        this.img = [];
    }

    function ImgStoreItem(n, s, w, h) {
        this.name = n;
        this.src = s;
        this.obj = null;

        if (s) {
            this.obj = new Image(w, h);
            this.obj.src = s;
        }
    }

    ImgStoreObject.prototype.find = function (name) {
        for (var i = 0; i <= this.count; i++) {
            if (this.img[i].name === name) {
                return i;
            }
        }

        return -1;
    };

    ImgStoreObject.prototype.add = function (n, s, w, h) {
        var i = this.find(n);

        if (i === -1) {
            i = ++this.count;
        }

        this.img[i] = new ImgStoreItem(n, s, parseInt(w, 10), parseInt(h, 10));
    };

    ImgStoreObject.prototype.getSrc = function (name) {
        var i = this.find(name);
        return i === -1 ? '' : this.img[i].src;
    };

    ImgStoreObject.prototype.getTag = function (name, iconID, altText) {
        var tag = '',
            i = this.find(name),
            src = i >= 0 ? this.img[i].src : '';

        if (src !== '') {
            tag = '<img src="' + src + '"';
            tag += (iconID !== '') ? ' id="' + iconID + '"' : '';
            tag += ' alt="' + ((altText) ? altText : '') + '" />';
        }

        return tag;
    };

    /*
     The TocEntry object.  This contains the data and methods for an individual toc item.
     */
    function TocEntry(owner, item, isLeaf, text, topicId, parentTopicId, nxtSib, prvSib, parEntry, depth) {
        this.owner = owner;
        this.isLeaf = !!isLeaf;
        this.type = isLeaf ? 'Document' : 'Folder';
        this.text = text;
        this.topicId = topicId;
        this.parentTopicId = parentTopicId;
        this.depth = depth;
        this.topicUrl = '../' + topicId + '.' + PROPS.HTML_EXT;
        this.thisIndex = item;
        //sequential traversal variables
        this.prevEntry = null;
        this.nextEntry = null;
        //tree traversal variables
        this.parent = parEntry;
        this.firstChild = null;
        //this.prevSibling = prvSib;
        this.nextSibling = nxtSib;
        this.domEl = null;
        this.isOpen = (owner === -1);
        this.isSelected = false;
        this.isDrawn = false;
    }

    TocEntry.prototype.getInnerHTML = function () {
        var o = this.owner,
            mRef = '="return ' + o.name,
            tmp = mRef + '.entries[' + this.thisIndex + '].',
            mOvr = ' onmouseover' + tmp + 'mouseOver(\'',
            mOut = ' onmouseout' + tmp + 'mouseOut(\'',
            iconTag = o.imgStore.getTag(this.getPMIconName(), 'plusMinusIcon' + this.thisIndex, ''),
            aLine = '',
            tip = this.text,
            theImg = o.imgStore.getTag(this.getDocIconName(), 'docIcon' + this.thisIndex, tip),
            sTxt = '<span class="' + (!this.isLeaf ? 'node' : 'leaf') + '">',
            lTxt = '<a id="tocentry' + this.thisIndex + '" ';

        if (!this.isLeaf) {
            aLine += '<a href="#" onclick' + mRef + '.toggleExpand(' + this.thisIndex + ',event);"' + mOvr +
                'plusMinusIcon\',this);"' + mOut + 'plusMinusIcon\');">' + iconTag + '</a>';
        } else {
            aLine += iconTag;
        }

        if (this.topicUrl !== '') {
            lTxt += ' href="' + this.topicUrl + '"' + mOvr + 'docIcon\',this);"' + mOut + 'docIcon\');"';
        }

        lTxt += tip ? ' title="' + tip + '">' : '>';
        aLine += sTxt + lTxt + theImg + this.text + '</a></span>';
        return aLine;
    };

    TocEntry.prototype.getPMIconName = function () {
        var n = 'icon' + (!this.isLeaf ? (this.isOpen ? 'Minus' : 'Plus') : 'Join');
        n += this === this.owner.firstEntry ? this.nextSibling == null ? 'Only' : 'Top' : this.nextSibling == null ? 'Bottom' : '';
        return n;
    };

    TocEntry.prototype.getDocIconName = function () {
        var is = this.owner.imgStore,
            n = this.type;

        n += (this.isOpen && (is.getSrc(n + 'Expanded') !== '')) ? 'Expanded' : '';
        n += (this.isSelected && (is.getSrc(n + 'Selected') !== '')) ? 'Selected' : '';

        return n;
    };

    TocEntry.prototype.setImg = function (imgID, imgName) {
        var img,
            o = this.owner,
            s = o.imgStore.getSrc(imgName);

        if (s !== '' && !o.isBusy) {
            img = dutils.getEl(document, imgID);

            if (img && img.src !== s) {
                img.src = s;
            }
        }
    };

    TocEntry.prototype.setIsOpen = function (isOpen) {
        isOpen = !!isOpen;
        if (this.isOpen !== isOpen && !this.isLeaf) {
            this.isOpen = isOpen;
            this.setImg('plusMinusIcon' + this.thisIndex, this.getPMIconName());
            this.setImg('docIcon' + this.thisIndex, this.getDocIconName());
            return true;
        } else {
            return false;
        }
    };

    TocEntry.prototype.setSelected = function (isSelected) {
        this.isSelected = isSelected;
        var s = dutils.getEl(document, 'tocentry' + this.thisIndex);

        if (s) {
            s.className = isSelected ? 'selected' : 'notselected';
        }
        /*
         if (s) {
         if (isSelected) {
         s.className = 'selected';
         }
         else {
         s.removeAttribute('class');
         }
         }
         */

        this.setImg('docIcon' + this.thisIndex, this.getDocIconName());

        if ((this.parent != null) && this.owner.selectParents) {
            this.parent.setSelected(isSelected);
        }
    };

    TocEntry.prototype.mouseOver = function (imgName) {
        var newImg = '';

        if (imgName === 'plusMinusIcon') {
            newImg = this.getPMIconName();
        } else if (imgName === 'docIcon') {
            newImg = this.getDocIconName();
        }
        //old school - move to stylesheet
        this.setImg(imgName + this.thisIndex, newImg + 'MouseOver');
        return true;
    };

    TocEntry.prototype.mouseOut = function (imgName) {
        var newImg = '';

        if (imgName === 'plusMinusIcon') {
            newImg = this.getPMIconName();
        } else if (imgName === 'docIcon') {
            newImg = this.getDocIconName();
        }

        this.setImg(imgName + this.thisIndex, newImg);
        return true;
    };

    /*
     Create and insert a new div node representing this toc entry into the dom tree
     at the correct location. Since we always create nodes sequentially this node's
     nextSibling and/or firstChild will not have been drawn yet, but its parent and/or
     previous item will have been drawn. If we have a previous item we add after it,
     otherwise we add after our parent item.
     */
    TocEntry.prototype.draw = function (containerDiv) {
        this.domEl = document.createElement('div');
        this.domEl.id = 'entryDIV' + this.thisIndex;
        this.domEl.className = 'tocItem';
        this.domEl.innerHTML = this.getInnerHTML();
        this.isDrawn = true;

        if (this.nextEntry != null && this.nextEntry.isDrawn) {
            containerDiv.insertBefore(this.domEl, this.nextEntry.domEl);
        } else {
            containerDiv.appendChild(this.domEl);
        }
    };

    function Toc() {
        this.count = -1;//entries array indexes are not necessarily sequential -- this is just the max
        this.size = 0;//the actual number of entries
        this.firstEntry = null;
        this.autoScrolling = true;
        this.name = 'theToc';
        this.contentFrame = 'topic';
        this.selectParents = false;
        this.selectedEntry = -1;
        this.isBusy = true;
        this.isVisible = true;
        this.imgStore = new ImgStoreObject;
        this.entries = [];
        this.contentDiv = null;//set after page loads when DOM is ready
        this.initOutlineIcons();
    }

    Toc.prototype.getEmptyEntryIndex = function () {
        for (var i = 0; i <= this.count; i++) {
            if (this.entries[i] == null) {
                break;
            }
        }

        if (i > this.count) {
            this.count = i;
        }

        return i;
    };

    Toc.prototype.addEntry = function (addTo, prevSib, isLeaf, text, topicId, parentTopicId, depth) {
        var eIndex = this.getEmptyEntryIndex();
        var newEntry = new TocEntry(this, eIndex, isLeaf, text, topicId, parentTopicId, null, prevSib, addTo, depth);
        this.entries[eIndex] = newEntry;

        if (prevSib != null) {
            prevSib.nextSibling = newEntry;
        }

        if (addTo != null) {
            if (prevSib == null) {
                addTo.firstChild = newEntry;
            }
        } else {
            this.firstEntry = newEntry;
        }

        newEntry.prevEntry = (prevSib != null) ? prevSib : addTo;

        if (newEntry.prevEntry != null) {
            newEntry.nextEntry = newEntry.prevEntry.nextEntry;
            newEntry.prevEntry.nextEntry = newEntry;
        }

        if (newEntry.nextEntry != null) {
            newEntry.nextEntry.prevEntry = newEntry;
        }

        newEntry.draw(this.contentDiv);//create the DOM node and add it to the document
        this.size++;
        return newEntry;
    };

    Toc.prototype.addChild = function (addTo, isLeaf, text, topicId, parentTopicId, depth) {
        var cEntry = null;

        if (addTo != null) {
            cEntry = addTo.firstChild;//get the first child of the entry we're adding to

            if (cEntry != null) { //we are the first child
                while (cEntry.nextSibling != null) {
                    cEntry = cEntry.nextSibling;
                }
            }
        }

        return this.addEntry(addTo, cEntry, isLeaf, text, topicId, parentTopicId, depth);
    };

    Toc.prototype.syncWithPage = function (topicId) {
        var eID;

        if (!this.isBusy && topicId) {
            eID = this.findEntry(topicId);

            //check if the toc item for the page is available
            if (eID >= 0) {
                this.selectEntry(eID);
                this.setEntry(eID, true);

                if (this.autoScrolling) {
                    this.refresh();
                    this.scrollTo(eID);
                }
            }
        }
    };

    Toc.prototype.refresh = function () {
        if (this.isBusy) {
            return;
        }

        var levelIndent = 17,
            nextItemArray = [],
            currEntry = this.firstEntry,
            level = (currEntry == null) ? 0 : 1,
            isVisible = true,
            lastVisibleLevel = 1,
            tocTopNode = dutils.getEl(document, 'tocTop'),
            yPos = tocTopNode ? tocTopNode.offsetHeight : 0,
            leftMargin = (PROPS.HELP_TYPE === "tablethtml") ? 9 : 0,
            divNode = null;

        while (level > 0) {
            divNode = currEntry.domEl;

            if (isVisible) {
                if (PROPS.RTL_TEXT) {
                    divNode.style.right = (currEntry.depth * levelIndent) + leftMargin + 'px';
                } else {
                    divNode.style.left = (currEntry.depth * levelIndent) + leftMargin + 'px';
                }

                divNode.style.top = yPos + 'px';//toc entry divs have style position:absolute - must set tops!
                divNode.style.visibility = 'visible';
                yPos += parseInt(divNode.offsetHeight, 10);
                lastVisibleLevel = level;
            } else {
                divNode.style.visibility = 'hidden';
                divNode.style.top = '0px';
            }

            if (currEntry.firstChild != null) { //if we have a child entry
                isVisible = (currEntry.isOpen === true) && isVisible;
                nextItemArray[level++] = currEntry.nextSibling;
                currEntry = currEntry.firstChild;
            } else { //look for our next sibling
                if (currEntry.nextSibling != null) {
                    currEntry = currEntry.nextSibling;
                } else {
                    while (level > 0) {
                        if (nextItemArray[--level] != null) {
                            currEntry = nextItemArray[level];
                            isVisible = (lastVisibleLevel >= level);
                            break;
                        }
                    }
                }
            }
        }
    };

    Toc.prototype.scrollTo = function (entryNo) {
        var scrBy, curBot, curTop, srBot, srTop,
            e = this.entries[entryNo];

        if (!e) {
            return;
        }

        srTop = e.domEl.style.top || e.domEl.style.pixelTop || e.domEl.offsetTop || 0;
        srTop = typeof srTop == 'string' ? parseInt(srTop) : srTop;
        srBot = srTop + 18;
        curTop = dutils.getScrollY();
        curBot = curTop + dutils.getClientHeight();

        if (srBot > curBot - 20 || srTop < curTop + 30) {
            scrBy = srBot - curBot + (curBot - curTop) / 2;

            if (srTop < curTop + scrBy) {
                scrBy = srTop - curTop - (curBot - curTop) / 2;
            }

            window.scrollBy(0, scrBy);
        }
    };

    Toc.prototype.selectEntry = function (entryNo) {
        var e;
        var oe = this.entries[this.selectedEntry];
        oe && oe.setSelected(false);

        e = this.entries[entryNo];
        e && e.setSelected(true);
        this.selectedEntry = entryNo;
    };

    Toc.prototype.setEntry = function (entryNo, state) {
        var cl = ',' + entryNo + ',';
        var e = this.entries[entryNo];
        var mc = e.setIsOpen(state);
        var p = e.parent;

        while (p != null) {
            cl += p.thisIndex + ',';
            mc |= p.setIsOpen(true);
            p = p.parent;
        }

        return mc;
    };

    Toc.prototype.findEntry = function (srchVal) {
        var e;

        if (srchVal) {
            for (var i = 0; i <= this.count; i++) {
                e = this.entries[i];

                if (e && e.topicId === srchVal) {
                    return i;
                }
            }
        }

        return -1;
    };

    Toc.prototype.toggleExpand = function (entryNo, e, fromClicked, state) {
        var centry;
        e = e || event;
        var shiftPressed = e.shiftKey;
        var entry = this.entries[entryNo];
        /*
         If this is not a leaf node -- and it shouldn't be if it had a toggle -- check
         to see if we need to load the node's children.
         */
        if (!entry.isLeaf) {
            if (typeof (state) == 'undefined') {
                state = !entry.isOpen;
            }

            if (entry.firstChild == null && state) {
                this.load(entry.topicId, entry.parentTopicId);
            }

            if (shiftPressed && entry.firstChild) {
                centry = entry.firstChild;

                while (centry) {
                    this.toggleExpand(centry.thisIndex, e, fromClicked, state);
                    centry = centry.nextSibling;
                }
            }

            this.setEntry(entryNo, state) && this.refresh();
        }

        return false;//called from event handlers
    };

    Toc.prototype.addLevel = function (entry, tocObj) {
        /*
         Add toc items if entry is defined but has no children or entry is undefined.
         Entry will be a toc item representing the root node of a toc section that needs
         to be loaded. If entry is undefined this will be the highest level toc section
         that we're adding.
         */
        var itemNode;
        var depth;
        var addToEntry;
        var parentTopicId;

        if (!entry || entry.firstChild == null) {
            addToEntry = null;
            depth = 0;
            /*
             First node is the root node in every toc.js file. We only add this for the
             root tocsection (when entry is undefined).
             */
            parentTopicId = tocObj["id"];

            if (!entry) { //this is the highest level 'book' item
                addToEntry = this.addChild(addToEntry, false, tocObj["title"], parentTopicId, null, depth);
            } else {
                addToEntry = entry;
                depth = entry.depth;
            }

            for (var i = 0; i < tocObj.entry.length; i++) {
                itemNode = tocObj.entry[i];
                this.addChild(addToEntry, itemNode['leaf'], itemNode['title'], itemNode['id'], parentTopicId, depth + 1);
            }
        }
    };

    /*
     Load toc entries if needed -- they may already be loaded.
     Return true if script loaded, false if not.

     Search for topic amongst current TOC entries.
     If not found load parent (recursively) - except if root topic.
     If entry has children load them.

     Called initially with topicId and parentId variables from topic files
     If parentTopicId is empty we're the root
     */
    Toc.prototype.load = function (topicId, parentTopicId, fromToggle) {
        var tmpParentTopicId = parentTopicId,
            parentTocObj = null,
            tocObj, tocObjLookup,
            that = this;
        /*
         Assign the callback called from the individual js toc files. Note the
         recursive call to this function. We're dynamically loading javascript
         to avoid cross frame/cross domain issues particularly when run from
         local file systems where modern browsers can be more restrictive.
         Since the js files are loaded asynchronously we ensure data is available
         (and the right ordering) by setting things up this way.
         */
        this.addJsTocSegment = function (tocSegmentObj) {
            tocSegmentObj.lookup = {};

            for (var i = 0; i < tocSegmentObj.entry.length; i++) {
                tocSegmentObj.lookup[tocSegmentObj.entry[i].id] = tocSegmentObj.entry[i];
            }

            that['toc_' + tocSegmentObj.id] = tocSegmentObj;
            that.load(topicId, parentTopicId, fromToggle);//close on these variables
        };

        while (tmpParentTopicId) { //ensure all js toc segments above us are loaded
            parentTocObj = this['toc_' + tmpParentTopicId]; //check for JS Object

            if (!parentTocObj) { //load JS file for tmpParentTopicId if not found
                dutils.loadJSFile('toc_' + tmpParentTopicId + '.js');
                return;
            } else {
                tmpParentTopicId = parentTocObj.parent;
            } //and check object's parent
        }

        tocObj = parentTopicId ? this['toc_' + parentTopicId] : null;
        tocObjLookup = tocObj ? tocObj.lookup[topicId] : null;

        // Check if we're a leaf - if not load our js file so we get our children in the toc
        if ((!tocObjLookup || !tocObjLookup.leaf) && !this['toc_' + topicId]) {
            dutils.loadJSFile('toc_' + topicId + '.js');
        } else if (this.loadEntries(topicId, parentTopicId, fromToggle)) {
            this.syncWithPage(topicId);
        } else {
            dutils.getEl(document, 'tocContent').innerHTML = "<div class=\'entry0\'>Contents not available.</div>" +
                "<div class=\'entry0\'>Unable to load toc.</div>";
        }
    };

    Toc.prototype.loadEntries = function (topicId, parentTopicId, fromToggle) {
        var noproblem = true,
            loadTocSegment = false,
            topicEntry = null,
            topicEntryId = topicId != null ? this.findEntry(topicId) : -1,
            tocObj, parentId, pEntry, pEntryId;

        if (topicEntryId === -1) { //topic not found among current entries in toc
            if (parentTopicId) { //recursive call to load parent
                noproblem = this.loadEntries(parentTopicId, null, fromToggle);

                if (noproblem) { //topic is in toc
                    topicEntryId = this.findEntry(topicId);//find it
                    topicEntry = this.entries[topicEntryId];
                    loadTocSegment = !topicEntry.isLeaf;
                }
            } else { //this is the root entry
                loadTocSegment = true;
            }
        } else { //topic is in toc
            topicEntry = this.entries[topicEntryId];

            if (!topicEntry.isLeaf && topicEntry.firstChild == null) {
                loadTocSegment = true;
            }
        }

        if (loadTocSegment) {
            tocObj = this['toc_' + topicId];

            if (tocObj) {
                parentId = tocObj.parent;
                pEntry = null;

                if (parentId) {
                    pEntryId = this.findEntry(parentId);
                    pEntry = (pEntryId !== -1) ? this.entries[pEntryId] : null;

                    if (pEntry == null || pEntry.firstChild == null) {
                        noproblem = this.loadEntries(parentId, null, fromToggle);//recursive call to load parent

                        if (noproblem) {
                            topicEntryId = this.findEntry(topicId);//now find it
                            topicEntry = this.entries[topicEntryId];
                        }
                    }
                }

                this.addLevel(topicEntry, tocObj);
                noproblem = true;
            } else {
                noproblem = false;
            }
        }

        return noproblem;
    };

    Toc.prototype.initOutlineIcons = function () {
        var store = this.imgStore,
            imgSize = 18,
            ip = PROPS.BOOKUI2SHAREDUI_RELPATH + 'images/',
            tabletPrefix = PROPS.HELP_TYPE === 'tablethtml' ? 't' : '';

        store.add('iconPlusTop', ip + 'plustop.png', imgSize, imgSize);
        store.add('iconPlus', ip + tabletPrefix + 'plus.png', imgSize, imgSize);
        store.add('iconPlusBottom', ip + tabletPrefix + 'plusbottom.png', imgSize, imgSize);
        store.add('iconPlusOnly', ip + 'plusonly.png', imgSize, imgSize);
        store.add('iconMinusTop', ip + 'minustop.png', imgSize, imgSize);
        store.add('iconMinus', ip + tabletPrefix + 'minus.png', imgSize, imgSize);
        store.add('iconMinusBottom', ip + tabletPrefix + 'minusbottom.png', imgSize, imgSize);
        store.add('iconMinusOnly', ip + tabletPrefix + 'minusonly.png', imgSize, imgSize);
        store.add('iconLine', ip + 'line.png', imgSize, imgSize);
        store.add('iconBlank', ip + 'blank.png', imgSize, imgSize);
        store.add('iconJoinTop', ip + 'jointop.png', imgSize, imgSize);
        store.add('iconJoin', ip + 'join.png', imgSize, imgSize);
        store.add('iconJoinBottom', ip + 'joinbottom.png', imgSize, imgSize);

        //Add folder and document images to the imgStore.
        store.add('Folder', ip + tabletPrefix + 'folder.png', imgSize, imgSize);
        store.add('FolderMouseOver', ip + tabletPrefix + 'folder_mo.png', imgSize, imgSize);
        store.add('FolderExpanded', ip + tabletPrefix + 'folder_ex.png', imgSize, imgSize);
        store.add('FolderExpandedMouseOver', ip + tabletPrefix + 'folder_ex_mo.png', imgSize, imgSize);
        store.add('FolderExpandedSelected', ip + tabletPrefix + 'folder_ex_sel.png', imgSize, imgSize);

        store.add('Document', ip + tabletPrefix + 'doc.png', imgSize, imgSize);
        store.add('DocumentMouseOver', ip + tabletPrefix + 'doc_mo.png', imgSize, imgSize);
        store.add('DocumentSelected', ip + tabletPrefix + 'doc_sel.png', imgSize, imgSize);
    };

    Toc.prototype.update = function (topicId, parentId) {
        /*
         We normally expect that the frameset has been loaded and that we're now trying
         to populate the content of the navigation frame with the toc content. This occurs
         when loading nav.html. This check for the existence of the topicframe tells
         us if we're in the frameset. We also need to handle loading nav.html without
         the frameset.
         */
        var topicframe = top.frames[this.contentFrame];

        if (!topicframe && (typeof (PROPS.MAIN_TOPIC) != 'undefined')) { //loading toc without frameset
            topicId = PROPS.MAIN_TOPIC.replace("." + PROPS.HTML_EXT, "");
        }

        this.load(topicId, parentId);
    };
}(DOM_UTILS_MODULE);

var theToc = TOC_MODULE.getToc();//global used in toc.js files

var NAV_MODULE = new function (toc, dutils) {
    var tocTab = null, searchTab = null, indexTab = null,
        tocDiv = null, searchDiv = null, indexDiv = null,
        indexScroll = 0,
        searchScroll = 0;

    //begin exported functions
    this.selectTab = function (node) {
        tabSelect(node);
    };
    this.updateToc = function (topicId, parentId) {
        tocUpdateFromTopic(topicId, parentId);
    };
    //end exported functions

    addEvent(window, 'load', onPageLoad);//register onload handler

    function activateTab(div, formName, savedScroll, inputId) {
        div.className = 'active';
        dutils.getEl(document, formName).className = 'active';//displays the form
        window.scrollTo(0, savedScroll);
        dutils.getEl(document, inputId).focus();
    }

    function deActivateTab(div, tab, formName) {
        tab.className = 'tab-unselected';
        div.className = 'inactive';

        if (arguments.length > 2) {
            dutils.getEl(document, formName).className = 'inactive';
        }
    }

    function tabSelect(tab) {
        var topicFrame = top.frames['topic'];

        if (tab.className === 'tab-unselected') {
            tab.className = 'tab-selected';

            if (tab.id === 'toctab') {
                toc.isVisible = true;
                tocDiv.className = 'active';
                topicFrame && postMsg(topicFrame, 'updateToc');
            } else {
                toc.isVisible = false;
                deActivateTab(tocDiv, tocTab);
            }

            if (indexDiv) {
                if (tab.id === 'indextab') { //indextab
                    activateTab(indexDiv, 'indexform', indexScroll, 'isearchfld');
                    typeof INDEX_MODULE === "undefined" && dutils.loadJSFile('index.js');
                } else {
                    if (indexDiv.className === 'active') { //save scroll location if we're deactivating index tab
                        indexScroll = dutils.getScrollY();
                    }

                    deActivateTab(indexDiv, indexTab, 'indexform');
                }
            }

            if (searchDiv) {
                if (tab.id === 'searchtab') {//loaded search.js if needed
                    activateTab(searchDiv, 'searchform', searchScroll, 'query');
                    typeof SEARCH_MODULE === "undefined" && dutils.loadJSFile('search.js');
                } else {
                    if (searchDiv.className === 'active') { //save scroll location if we're deactivating search tab
                        searchScroll = dutils.getScrollY();
                    }

                    deActivateTab(searchDiv, searchTab, 'searchform');
                }
            }
        }
    }

    /*
     tocUpdateFromTopic is called from a content page in the topic frame when it's loaded.
     */
    function tocUpdateFromTopic(topicId, parentId) {
        if (!toc || toc.isBusy) {
            setTimeout(function () {
                tocUpdateFromTopic(topicId, parentId);
            }, 200);
        } else {
            toc.isVisible && toc.update(topicId, parentId);
        }
    }

    function getTabWidth() {
        return (tocTab ? tocTab.scrollWidth : 0) + (indexTab ? indexTab.scrollWidth : 0) + (searchTab ? searchTab.scrollWidth : 0);
    }

    function onPageLoad() {
        var frameset, tabwidth,
            topicframe = top.frames['topic'];

        tocTab = dutils.getEl(document, 'toctab');
        searchTab = PROPS.HAS_SEARCH && dutils.getEl(document, 'searchtab');
        indexTab = PROPS.HAS_INDEX && dutils.getEl(document, 'indextab');
        tocDiv = dutils.getEl(document, 'toc');
        indexDiv = PROPS.HAS_INDEX && dutils.getEl(document, 'index');
        searchDiv = PROPS.HAS_SEARCH && dutils.getEl(document, 'search');
        toc.contentDiv = dutils.getEl(document, 'tocContent');

        toc.isBusy = false;
        //cross frame call will call back to tocUpdateFromTopic
        topicframe && postMsg(topicframe, 'updateToc');
        tocTab.innerHTML = PROPS.CONTENTS_STR;

        if (searchTab) {
            searchTab.innerHTML = PROPS.SEARCH_STR;
            dutils.getEl(document, 'searchbtn').value = PROPS.SEARCH_STR;
            searchTab.className = 'tab-unselected';
        }

        if (indexTab) {
            indexTab.innerHTML = PROPS.INDEX_STR;
            indexTab.className = "tab-unselected";
        }

        /*
         Minor cosmetic correction as IE will drop the tabs down
         a bit when whitespace is nowrap on the div container for the tabs.
         */
        if (document.all) {
            dutils.getEl(document, 'tabs').style.whiteSpace = 'normal';
        }

        if (!PROPS.RTL_TEXT && (PROPS.HELP_TYPE !== "tablethtml")) {
            /*
             Resize nav frame so we initially see all the tabs.
             */
            tabwidth = getTabWidth();

            if (tabwidth > 230) {
                frameset = dutils.getEl(top.document, 'frameset');

                if (frameset) {
                    frameset.cols = tabwidth + 22 + ",*";
                }
            }
        }
    }

    function addEvent(obj, evt, fn) {
        if (obj.addEventListener) {
            obj.addEventListener(evt, fn, false);
        } else if (obj.attachEvent) {
            obj.attachEvent('on' + evt, fn);
        }
    }

    window['postMessage'] && addEvent(window, "message", getMsg);

}(theToc, DOM_UTILS_MODULE);